@@ -3,7 +3,9 @@ from __future__ import generator_stop
 import sys
 from collections.abc import Sequence
 from time import sleep
-from typing import Any, Dict, List, Optional, Set, TextIO, Tuple
+from typing import Any, Dict, List, Optional
+from typing import Sequence as SequenceT
+from typing import Set, TextIO, Tuple
 
 from aria2p import Client
 from aria2p.client import DEFAULT_HOST, DEFAULT_PORT, ClientException
@@ -251,11 +253,15 @@ class AriaDownloader:
         uri: str,
         path: Optional[str] = None,
         filename: Optional[str] = None,
-        headers: Optional[Sequence[str]] = None,
+        headers: Optional[SequenceT[str]] = None,
         max_connections: Optional[int] = None,
         split: Optional[int] = None,
         continue_: Optional[bool] = None,
         retry_wait: Optional[int] = None,
+        max_tries: Optional[int] = None,
+        connect_timeout: Optional[int] = None,
+        timeout: Optional[int] = None,
+        no_netrc: Optional[bool] = None,
     ) -> str:
 
         """Downloads `uri` to directory `path`.
@@ -271,11 +277,15 @@ class AriaDownloader:
             {
                 "dir": path,
                 "out": filename,
+                "header": headers,
                 "max-connection-per-server": max_connections,
                 "split": split,
                 "continue": aria_bool(continue_),
                 "retry-wait": retry_wait,
-                "header": headers,
+                "max-tries": max_tries,
+                "connect-timeout": connect_timeout,
+                "timeout": timeout,
+                "no-netrc": aria_bool(no_netrc),
             },
         )
 
@@ -289,14 +299,31 @@ class AriaDownloader:
         uri: str,
         path: Optional[str] = None,
         filename: Optional[str] = None,
-        headers: Optional[Sequence[str]] = None,
+        headers: Optional[SequenceT[str]] = None,
         max_connections: Optional[int] = None,
         split: Optional[int] = None,
         continue_: Optional[bool] = None,
         retry_wait: Optional[int] = None,
+        max_tries: Optional[int] = None,
+        connect_timeout: Optional[int] = None,
+        timeout: Optional[int] = None,
+        no_netrc: Optional[bool] = None,
     ) -> Optional[Tuple[str, str, str]]:
 
-        queued_gid = self.download(uri, path, filename, headers, max_connections, split, continue_, retry_wait)
+        queued_gid = self.download(
+            uri,
+            path,
+            filename,
+            headers,
+            max_connections,
+            split,
+            continue_,
+            retry_wait,
+            max_tries,
+            connect_timeout,
+            timeout,
+            no_netrc,
+        )
         if self.managed_downloads() >= num:
             finished_gid, path = self.block_one()
             return queued_gid, finished_gid, path
