@@ -4,8 +4,7 @@ from itertools import cycle
 from typing import Iterable, Iterator
 
 
-def xor_stream(it, key, cyclekey=True):
-    # type: (Iterable[bytes], Iterable[bytes], bool) -> Iterator[bytes]
+def xor_stream(it: Iterable[bytes], key: Iterable[bytes], cyclekey: bool = True) -> Iterator[bytes]:
 
     if cyclekey:
         key = cycle(key)
