@@ -88,8 +88,7 @@ def _islink(path):
     return FileAttributes & FILE_ATTRIBUTE_REPARSE_POINT == FILE_ATTRIBUTE_REPARSE_POINT
 
 
-def _uncabspath(path):
-    # type: (str, )-> str
+def _uncabspath(path: str) -> str:
 
     if path.startswith(unc_prefix):
         return os.path.abspath(path)
